@@ -5,7 +5,7 @@ from collections import Iterable
 
 import pandas as pd
 from longling import as_list
-from numpy.random import RandomState
+from numpy.random import default_rng
 from tqdm import tqdm
 
 """
@@ -16,12 +16,12 @@ contrastive triplet
 (query, pos, neg)
 """
 
-__all__ = ["TripletPairSampler", "UserSpecificPairSampler", "ItemSpecificSampler"]
+__all__ = ["Sampler", "TripletPairSampler", "UserSpecificPairSampler", "ItemSpecificSampler"]
 
 
 class Sampler(object):
     def __init__(self, random_state=10, pad_value=0):
-        self.random_state = RandomState(random_state)
+        self.random_state = default_rng(random_state)
         self.pad_value = pad_value
 
     def padding(self, sampled: list, n, *args, **kwargs) -> list:
@@ -67,13 +67,19 @@ class TripletPairSampler(Sampler):
     >>> sampler(0, 3, padding=False)
     (0, [])
     >>> sampler(0, 5, padding=False, implicit=True)
-    (3, [0, 3, 2])
+    (3, [2, 3, 0])
     >>> sampler(0, 5, padding=False, implicit=True, excluded_key=[3])
-    (2, [2, 0])
+    (2, [0, 2])
     >>> sampler(0, 5, padding=True, implicit=True, excluded_key=[3])
-    (2, [0, 2, 0, 0, 0])
+    (2, [2, 0, 0, 0, 0])
     >>> sampler(0, 5, implicit=True, pad_value=-1)
-    (3, [3, 2, 0, -1, -1])
+    (3, [2, 3, 0, -1, -1])
+    >>> sampler(0, 5, implicit=True, fast_implicit=True, pad_value=-1)
+    (3, [0, 2, 3, -1, -1])
+    >>> sampler(0, 5, implicit=True, fast_implicit=True, with_n_implicit=3, pad_value=-1)
+    (3, [0, 2, 3, -1, -1, -1, -1, -1])
+    >>> sampler(0, 5, implicit=True, fast_implicit=True, with_n_implicit=3, pad_value=-1, padding_implicit=True)
+    (3, [0, 2, 3, -1, -1, -1, -1, -1])
     >>> rating_matrix = pd.DataFrame({
     ...     "query": [0, 1, 1, 1, 2],
     ...     "key": [1, 3, 0, 2, 1],
@@ -92,7 +98,7 @@ class TripletPairSampler(Sampler):
     2            []  [1]
     >>> sampler = TripletPairSampler(triplet_df, "query", query_range=3, key_range=4)
     >>> sampler([0, 1, 2], 5, implicit=True, pad_value=-1)
-    [(3, [0, 3, 2, -1, -1]), (2, [3, 1, -1, -1, -1]), (4, [2, 1, 0, 3, -1])]
+    [(3, [2, 3, 0, -1, -1]), (1, [1, -1, -1, -1, -1]), (3, [3, 0, 2, -1, -1])]
     >>> sampler([0, 1, 2], 5, pad_value=-1)
     [(0, [-1, -1, -1, -1, -1]), (1, [3, -1, -1, -1, -1]), (1, [1, -1, -1, -1, -1])]
     >>> sampler([0, 1, 2], 5, neg=False, pad_value=-1)
@@ -146,7 +152,41 @@ class TripletPairSampler(Sampler):
             self.df.set_index(self.query_field, inplace=True)
 
     def __call__(self, query: (int, str, list), n=1, excluded_key=None, neg=True, implicit=False, padding=True,
-                 *args, verbose=True, return_column=False, split_sample_to_column=False, **kwargs):
+                 *args, verbose=True, return_column=False, split_sample_to_column=False,
+                 padding_implicit=False, fast_implicit=False, with_n_implicit=None, **kwargs):
+        """
+        To get only explicit neg (pos) samples, set implicit as False, padding as False,
+        padding_implicit as False and with_n_implicit as None.
+
+        To get at maximum of n samples, where the insufficient ones are implicit sampled,
+        set padding and padding_implicit as True, and with_n_implicit as None.
+
+        To get explicit samples with at maximum of n implicit samples,
+        set implicit as False, with_n_implicit as n.
+
+        Parameters
+        ----------
+        query
+        n
+        excluded_key
+        neg: bool
+            negative sampling (set neg as True) or positive sampling (set neg as False)
+        implicit: bool
+            explicit sampling (set implicit as False) or implicit sampling (set implicit as True)
+        padding
+        args
+        verbose
+        return_column
+        split_sample_to_column
+        padding_implicit
+        fast_implicit
+        with_n_implicit
+        kwargs
+
+        Returns
+        -------
+
+        """
         if isinstance(query, (int, str)):
             sample = self.sample(
                 query=query,
@@ -157,17 +197,42 @@ class TripletPairSampler(Sampler):
                 query=query,
                 n=n,
                 excluded_key=excluded_key,
-                neg=neg
+                fast_mode=fast_implicit
             )
             n_sample = len(sample)
-            if padding:
-                return n_sample, self.padding(sample, n, **kwargs)
+            padding_n = n
+            if with_n_implicit is not None:
+                padding_n += with_n_implicit
+                implicit_sample = self.implicit_sample(
+                    query=query, n=with_n_implicit, excluded_key=excluded_key,
+                    samples=sample,
+                    fast_mode=fast_implicit)
+                n_sample += len(implicit_sample)
+                sample += implicit_sample
+
+            if padding is True:
+                if padding_implicit is True:
+                    if n_sample < padding_n:
+                        padding_sample = self.implicit_sample(query=query, n=padding_n - n_sample,
+                                                              excluded_key=excluded_key, samples=sample,
+                                                              fast_mode=fast_implicit)
+                        n_padding_sample = len(padding_sample)
+                        return n_sample + n_padding_sample, self.padding(sample + padding_sample, padding_n, **kwargs)
+                else:
+                    return n_sample, self.padding(sample, padding_n, **kwargs)
+
             return n_sample, sample
         else:
             neg = neg if isinstance(neg, Iterable) else [neg] * len(query)
             excluded_key = excluded_key if isinstance(excluded_key, Iterable) else [excluded_key] * len(query)
             n_and_sample = [
-                self(_query, n, _excluded_key, _neg, implicit, padding, *args, **kwargs)
+                self(
+                    _query, n, _excluded_key, _neg, implicit, padding,
+                    *args,
+                    padding_implicit=padding_implicit,
+                    fast_implicit=fast_implicit,
+                    with_n_implicit=with_n_implicit,
+                    **kwargs)
                 for _query, _excluded_key, _neg in tqdm(
                     zip(query, excluded_key, neg), "sampling", disable=not verbose
                 )
@@ -186,15 +251,30 @@ class TripletPairSampler(Sampler):
             candidates = list(set(candidates) - set(as_list(excluded_key)))
 
         sampled = self.random_state.choice(candidates, min(n, len(candidates)), replace=False).tolist()
+        # rs = default_rng(10)
+        # sampled = rs.choice(candidates, min(n, len(candidates)), replace=False).tolist()
         return sampled
 
-    def implicit_sample(self, query: (int, str, list), n=1, excluded_key=None, neg=True, *args, **kwargs):
-        exclude = set(self.df.loc[query][self.pos_field] if neg else self.df.loc[query][self.neg_field])
+    def implicit_sample(self, query: (int, str, list), n=1, excluded_key=None, fast_mode=False, samples=None,
+                        *args, fast_max_try=100, **kwargs):
+        exclude = set(self.df.loc[query][self.pos_field]) | set(self.df.loc[query][self.neg_field])
         if excluded_key is not None:
             exclude |= set(as_list(excluded_key))
-        candidates = list(set(range(*self.key_range)) - exclude)
-        sampled = self.random_state.choice(candidates, min(n, len(candidates)), replace=False).tolist()
-        return sampled
+        if samples is not None:
+            exclude |= set(samples)
+        if fast_mode is False:
+            candidates = list(set(range(*self.key_range)) - exclude)
+            sampled = self.random_state.choice(candidates, min(n, len(candidates)), replace=False).tolist()
+            return sampled
+        else:
+            sampled = set()
+            try_cnt = 0
+            while len(sampled) < n and try_cnt < n + fast_max_try:
+                _sample = self.random_state.integers(*self.key_range)
+                if _sample not in exclude and _sample not in sampled:
+                    sampled.add(_sample)
+                try_cnt += 1
+            return list(sampled)
 
     @staticmethod
     def rating2triplet(rating_matrix: pd.DataFrame, query_field, key_field, value_field=None,
@@ -245,13 +325,13 @@ class UserSpecificPairSampler(TripletPairSampler):
     (0, [0])
     >>> sampler = UserSpecificPairSampler(triplet_df, item_id_range=item_num)
     >>> sampler(0, implicit=True)
-    (1, [0])
+    (1, [3])
     >>> sampler(0, 5, implicit=True)
     (3, [3, 2, 0, 0, 0])
     >>> sampler(0, 5, implicit=True, pad_value=-1)
     (3, [3, 2, 0, -1, -1])
     >>> sampler([0, 1, 2], 5, implicit=True, pad_value=-1)
-    [(3, [3, 2, 0, -1, -1]), (1, [1, -1, -1, -1, -1]), (3, [3, 0, 2, -1, -1])]
+    [(3, [2, 3, 0, -1, -1]), (1, [1, -1, -1, -1, -1]), (3, [2, 0, 3, -1, -1])]
     >>> rating_matrix = pd.DataFrame({
     ...     "user_id": [0, 1, 1, 1, 2],
     ...     "item_id": [1, 3, 0, 2, 1],
@@ -271,13 +351,13 @@ class UserSpecificPairSampler(TripletPairSampler):
     [(1, [1, -1, -1, -1, -1]), (2, [0, 2, -1, -1, -1]), (0, [-1, -1, -1, -1, -1])]
     >>> sampler(rating_matrix["user_id"], 2, neg=rating_matrix["score"],
     ...     excluded_key=rating_matrix["item_id"], pad_value=-1)
-    [(0, [-1, -1]), (2, [0, 2]), (1, [3, -1]), (1, [3, -1]), (0, [-1, -1])]
+    [(0, [-1, -1]), (2, [2, 0]), (1, [3, -1]), (1, [3, -1]), (0, [-1, -1])]
     >>> sampler(rating_matrix["user_id"], 2, neg=rating_matrix["score"],
     ...     excluded_key=rating_matrix["item_id"], pad_value=-1, return_column=True)
-    ((0, 2, 1, 1, 0), ([-1, -1], [2, 0], [3, -1], [3, -1], [-1, -1]))
+    ((0, 2, 1, 1, 0), ([-1, -1], [0, 2], [3, -1], [3, -1], [-1, -1]))
     >>> sampler(rating_matrix["user_id"], 2, neg=rating_matrix["score"],
     ...     excluded_key=rating_matrix["item_id"], pad_value=-1, return_column=True, split_sample_to_column=True)
-    ((0, 2, 1, 1, 0), [(-1, 0, 3, 3, -1), (-1, 2, -1, -1, -1)])
+    ((0, 2, 1, 1, 0), [(-1, 2, 3, 3, -1), (-1, 0, -1, -1, -1)])
     """
 
     def __init__(self, triplet_df: pd.DataFrame,
@@ -339,13 +419,13 @@ class ItemSpecificSampler(TripletPairSampler):
     (0, [0])
     >>> sampler = ItemSpecificSampler(triplet_df, user_id_range=user_num)
     >>> sampler(0, implicit=True)
-    (1, [0])
+    (1, [2])
     >>> sampler(0, 5, implicit=True)
-    (2, [0, 2, 0, 0, 0])
+    (2, [2, 0, 0, 0, 0])
     >>> sampler(0, 5, implicit=True, pad_value=-1)
-    (2, [2, 0, -1, -1, -1])
+    (2, [0, 2, -1, -1, -1])
     >>> sampler([0, 1, 2], 5, implicit=True, pad_value=-1)
-    [(2, [0, 2, -1, -1, -1]), (1, [1, -1, -1, -1, -1]), (2, [2, 0, -1, -1, -1])]
+    [(2, [0, 2, -1, -1, -1]), (1, [1, -1, -1, -1, -1]), (2, [0, 2, -1, -1, -1])]
     >>> rating_matrix = pd.DataFrame({
     ...     "user_id": [0, 1, 1, 1, 2],
     ...     "item_id": [1, 3, 0, 2, 1],
@@ -374,6 +454,7 @@ class ItemSpecificSampler(TripletPairSampler):
     ...     excluded_key=rating_matrix["user_id"], pad_value=-1, return_column=True, split_sample_to_column=True)
     ((1, 0, 0, 0, 1), [(2, -1, -1, -1, 0), (-1, -1, -1, -1, -1)])
     """
+
     def __init__(self, triplet_df: pd.DataFrame,
                  query_field="item_id", pos_field="pos", neg_field="neg", set_index=False,
                  item_id_range=None, user_id_range=None,
