@@ -1,11 +1,11 @@
 # coding: utf-8
 # created by tongshiwei on 18-2-5
 import math
+import sys
 import warnings
 from collections import OrderedDict
 from contextlib import contextmanager
 
-import sys
 from longling import Timer
 from longling.lib import ProgressMonitor, IterableMIcing
 from longling.lib.stream import flush_print
@@ -49,7 +49,7 @@ class ConsoleProgressMonitorPlayer(object):
             __indexes = ["%s-%s" % (prefix, _index) for _index in _indexes]
             arguments += __indexes
             index_header += (" " * 2).join(
-                ["{:>%s}" % min(len(index), 15) for index in __indexes]
+                ["{:>%s}" % max(len(index), 15) for index in __indexes]
             )
 
         self.output_formatter = self.info_header + " " * 2 + index_header + " " * 2 + self.progress_header_fmt
@@ -231,7 +231,9 @@ class EpisodeCPMP(ConsoleProgressMonitorPlayer):
 
 
 PLAYER_TYPE = {
+    None: ConsoleProgressMonitorPlayer,
     "default": ConsoleProgressMonitorPlayer,
+    "simple": ConsoleProgressMonitorPlayer,
     "epoch": EBCPMP,
     "episode": EpisodeCPMP,
 }
