@@ -232,6 +232,12 @@ class Configuration(object):
     {'a': 1, 'b': 'example', 'c': [0, 2], 'd': {'a1': 3}}
     >>> c.default_file_format()
     'json'
+    >>> c.get("a")
+    1
+    >>> c.get("e") is None
+    True
+    >>> c.get("e", 0)
+    0
     """
 
     def __init__(self, logger=logging, **kwargs):
@@ -362,7 +368,13 @@ class Configuration(object):
         return getattr(self, item)
 
     def __contains__(self, item):
-        return item in self.class_var
+        return item in vars(self)
+
+    def get(self, k, default=None):
+        if k in self:
+            return self[k]
+        else:
+            return default
 
     def items(self):
         return {k: v for k, v in self.parsable_var.items()}
@@ -382,7 +394,7 @@ class Configuration(object):
             所有非参变量
         """
         return CLASS_EXCLUDE_NAMES | {
-            'class_var', 'parsable_var', 'items', 'load', 'dump', 'help_info'
+            'class_var', 'parsable_var', 'items', 'load', 'dump', 'help_info', 'get'
         } | cls.run_time_variables()
 
     @classmethod
